//------------------------------------------------------------------------------//
//				AXC-PL01_SAMPLE1_READ MODULE  V1.20					//
//			CompactFlash CF+ Programmable Logic CARD					//
//					ADTEK SYSTEM SCIENCE CO.,LTD.						//
//				Since 2005/01/13  LAST UPDATE 2005/11/07				//
//		     			<< S1_READ U3 >>							//
//------------------------------------------------------------------------------//

//------------------------------------------------------------------------------//
//	Module	C_ -> CompactFlash, L_ ->LocalSignal, P -> PIO Signal			//
//			n  -> Active Low Signal									//
//------------------------------------------------------------------------------//

module S1_READ

	(	IO_nCE_I,
		IO_nREG_I,
		COR_IOENB_I,
		ENIO_nIORD_I,
		IO_C_ADR_I,
		MEM_nOE,
		IO_DATAIN_I,
		IO_DATAOUT_EN_I,
		COR_REG_I,
		C_RESET,
		C_DATAOUT_O,
		C_DATAOUT_EN_O,
		IO_nINPACK_O,		 
		IO_nIOIS16_O

	);


//-----------------------Input Signal Definition--------------------------------//

	input 	[2:1]IO_nCE_I;					// CompactFlash Card Enable
										// Card Bus @U1,U3,U4

	input 	IO_nREG_I;					// CompactFlash Register Access
										// Card Bus  U0,U1,U3,U4

  	input 	COR_IOENB_I;					// Configuration Option Register
										// Enable Bit[0] 200H <- 01H
										// U0  MAIN,U3

	input	ENIO_nIORD_I;					// I/O Read Enable M
										// MAIN  U3
												
 	input	[10:0]IO_C_ADR_I;				// CompactFlash ADDRESS
										// Card Bus  U0,U1,U3,U4

	input	MEM_nOE;						// CompactFlash Memory Output Enable
										// Card Bus  U0,U3

	input 	[13:0]IO_DATAIN_I;				// OIO͂DATA
										// U2  U3

	input	[13:0]IO_DATAOUT_EN_I;			// I/O Select Register o
										// U1  U2,U3 
					     
	input	[7:0]COR_REG_I;				// Configuration Option Register
										// U0  U3

	input 	C_RESET;						// CompactFlash RESET (Active"H")
										// Card Bus  U0,U3


//-----------------------Output Signal Definition-------------------------------//


	output 	[13:0]C_DATAOUT_O;				// CARD DATA OUT
										// U3  U2

	output 	[1:0]C_DATAOUT_EN_O;			// CompactFlash DATA OUT Enable
										// U3  MAIN,U2

	output 	IO_nINPACK_O;					// CompactFlash InputAcknowledge
										// U3  Card Bus
											 
 	output 	IO_nIOIS16_O;					// CompactFlash Writeprotect or IO=16bit
										// U3  Card Bus 

//------------------------Register Definition-----------------------------------//

 	reg		[13:0]C_DATAOUT_O;				// CARD DATA OUT Register

	reg		[1:0]C_DATAOUT_EN_O;			// CompactFlash DATA OUT Enable Register

	reg		IO_nINPACK_O;					// CompactFlash InputAcknowledge Register

	reg		IO_nIOIS16_O;					// CompactFlash Writeprotect or IO=16bit Register


//------------------------------------------------------------------------------//

//------------------------------------------------------------------------------//
//																//
//																//
//	[10:0]a9_oe_en_adr_iord_reg_ce									//
//																//
//	10 9 8  7 6 5 4  3 2 1 0											//
//	 | | |  | | | |  | | | |											//
//	 | | |  | | | |  | | | +- IO_nCE_I[1]								//
//	 | | |  | | | |  | | +--- IO_nCE_I[2]								//
//	 | | |  | | | |  | +----- IO_nREG_I								//
//	 | | |  | | | |  +------- ENIO_nIORD_I								//
//	 | | |  | | | |												//
//	 | | |  | | | +---------- IO_C_ADR_I[0]								//
//	 | | |  | | +------------ IO_C_ADR_I[1]								//
//	 | | |  | +-------------- IO_C_ADR_I[2]								//
//	 | | |  +---------------- IO_C_ADR_I[3]								//
//	 | | |														//
//	 | | +------------------- COR_IOENB_I								//
//	 | +--------------------- MEM_nOE									//
//	 +----------------------- (~IO_C_ADR_I[10]&IO_C_ADR_I[9]&~IO_C_ADR_I[8])	//
//																//
//------------------------------------------------------------------------------//

	wire		[10:0]a9_oe_en_adr_iord_reg_ce;
	assign	a9_oe_en_adr_iord_reg_ce[1:0]	= IO_nCE_I[2:1];
	assign	a9_oe_en_adr_iord_reg_ce[2]	= IO_nREG_I;
	assign	a9_oe_en_adr_iord_reg_ce[3]	= ENIO_nIORD_I;
	assign	a9_oe_en_adr_iord_reg_ce[7:4]	= IO_C_ADR_I[3:0];
	assign	a9_oe_en_adr_iord_reg_ce[8]	= COR_IOENB_I;
	assign	a9_oe_en_adr_iord_reg_ce[9]	= MEM_nOE;
	assign	a9_oe_en_adr_iord_reg_ce[10]	= (~IO_C_ADR_I[10]&IO_C_ADR_I[9]&~IO_C_ADR_I[8]);


//------------------------------------------------------------------------------//
//	Compact Flash CARD OUTPUT DATA IW[						//
//																//
//------------------------------------------------------------------------------//

	always@ (C_RESET
			or IO_DATAIN_I
			or IO_DATAOUT_EN_I 
 			or a9_oe_en_adr_iord_reg_ce 
			or COR_REG_I ) begin

  		if (C_RESET == 1) begin
			C_DATAOUT_O    <= 14'b11_1111_1111_1111;

		end else begin

			casex (a9_oe_en_adr_iord_reg_ce[10:0])

				11'bx11_000x_xx0x : C_DATAOUT_O 	<= IO_DATAIN_I[13:0];
				11'bx11_0000_xx10 : C_DATAOUT_O	<= IO_DATAIN_I[7:0];
				11'bx11_0001_xx10 : C_DATAOUT_O	<= IO_DATAIN_I[13:8];

				11'bx11_001x_xx0x : C_DATAOUT_O	<= IO_DATAOUT_EN_I[13:0];
				11'bx11_0010_xx10 : C_DATAOUT_O	<= IO_DATAOUT_EN_I[7:0];
				11'bx11_0011_xx10 : C_DATAOUT_O	<= IO_DATAOUT_EN_I[13:8];	

	  			11'b10x_0000_10xx : C_DATAOUT_O	<= COR_REG_I[7:0];

  				default		   : C_DATAOUT_O	<= 14'b11_1111_1111_1111;

	 		endcase 
		end
	end

//------------------------------------------------------------------------------//
//	CompactFlash DATA OUT Enable Register	 							//
//																//
//------------------------------------------------------------------------------//

	always@ (C_RESET or a9_oe_en_adr_iord_reg_ce) begin

		if (C_RESET == 1) begin
			C_DATAOUT_EN_O <= 2'b00;

		end else begin
  
			casex (a9_oe_en_adr_iord_reg_ce[10:0])

		  		11'bx11_xxx0_0000 	: C_DATAOUT_EN_O <= 2'b11;
		  		11'bx11_xxxx_0010 	: C_DATAOUT_EN_O <= 2'b01; 
		  		11'bx11_xxxx_0001 	: C_DATAOUT_EN_O <= 2'b10; 
		  		11'b10x_0000_10xx 	: C_DATAOUT_EN_O <= 2'b01;
				 
		  		default			: C_DATAOUT_EN_O <= 2'b00; 

			endcase 
		end
	end

//------------------------------------------------------------------------------//
//	CompactFlash InputAcknowledge (INPACK#) `							//
//																//
//------------------------------------------------------------------------------//

	always@ (C_RESET or a9_oe_en_adr_iord_reg_ce) begin

		if (C_RESET == 1) begin
			IO_nINPACK_O   <= 1'b1;

		end else begin

			casex (a9_oe_en_adr_iord_reg_ce[10:0])

		  		11'bxx1_xxxx_00xx	:IO_nINPACK_O	<= 1'b0;

		  		default			:IO_nINPACK_O	<= 1'b1; 

			endcase 
		end
	end

//------------------------------------------------------------------------------//
//	CompactFlash Writeprotect or IO=16bit Register (IOIS16#) `			//
//																//
//------------------------------------------------------------------------------//

	always@ (C_RESET or a9_oe_en_adr_iord_reg_ce) begin

		if (C_RESET == 1) begin
			IO_nIOIS16_O	<= 1'b1;

		end else begin

			casex (a9_oe_en_adr_iord_reg_ce[10:0])

		  		11'bxx1_xxx0_x0xx	: IO_nIOIS16_O	<= 1'b0;

		  		default			: IO_nIOIS16_O	<= 1'b1; 

			endcase 
		end
	end 

//------------------------------------------------------------------------------//

endmodule

