//------------------------------------------------------------------------------//
//				AXC-PL01_SAMPLE3_REGISTER Module  V1.20					//
//			CompactFlash CF+ Programmable Logic CARD					//
//					ADTEK SYSTEM SCIENCE CO.,LTD.						//
//				Since 2005/01/13  LAST UPDATE 2005/11/07				//
//						<< S3_REGISTER U1 >>						//
//------------------------------------------------------------------------------//

//------------------------------------------------------------------------------//
//	Module	C_ -> CompactFlash, L_ ->LocalSignal, P -> PIO Signal			//
//			n  -> Active Low Signal									//
//------------------------------------------------------------------------------//

module S3_REGISTER

	(	C_DATAIN_I,
		IO_C_ADR_I,
		IO_nCE_I,
		IO_nREG_I,
		ENIO_nIOWR_I,
		MRST,
		L_125GEN,
		IO_DATAOUT_O,
		IO_DATAOUT_EN_O,
		IO_DATAOUT_SEL_O,
		IO_PU_O
	);


//-----------------------Input Signal Definition--------------------------------//

	input	[13:0] C_DATAIN_I;				//  CompactFlash DATA Buses
										// (IO Buffer CompactFlash CARD DATA)
										// U2  U0,U1,U4

	input	[3:0] IO_C_ADR_I;				// CompactFlash ADDRESS
										// Card Bus  U0,U1,U3,U4

	input	[2:1] IO_nCE_I;				// CompactFlash ChipEnable "H"Byte/"L"Byte
										// Card Bus @U1,U3,U4

	input	IO_nREG_I;					// CompactFlash Register Access
										// Card Bus  U0,U1,U3,U4

	input	ENIO_nIOWR_I;					// I/O Write Enable M
										// MAIN  U1

	input	MRST;						// RtBM[VZbgM
										// MAIN  U1,U4

	input	L_125GEN;						// Write Mode Switch
										// MAIN  U1,U2


//-----------------------Output Signal Definition-------------------------------//


	output	[13:0] IO_DATAOUT_O;			// PIO DATA IO
										// OIOւOUTDATA o
										// U1  U2

	output	[13:0] IO_DATAOUT_EN_O;			// PIO Output Enable
										// OIOւOUTDATA Enable o
										// U1  U2,U3

	output	[13:0] IO_DATAOUT_SEL_O;			// PIO OUTPUT Sellect(OD or OUT)
										// U1  U2

	output	[13:0] IO_PU_O;				// PIO DATA_PUSH/PULL Out
										// U1  PIO Side Bus

//-----------------------Register Definition------------------------------------//


	reg		[13:0] IO_DATAOUT_O;			// PIO DATA IO Register

	reg		[13:0] IO_DATAOUT_EN_O;			// PIO Output Enable Register

	reg		[13:0] IO_DATAOUT_SEL_O;			// PIO OUTPUT Sellect(OC or OUT) Register

	reg		[13:0] IO_PU_O_M;				// PIO DATA_PUSH/PULL Out Register								   	


//------------------------------------------------------------------------------//
//	JTAGC@LI/Oݒ											//
//		:Write Mode SwitchݐݒɂȂĂꍇɁALM	//
//			Hizɐݒ肵܂B										//
//				[13:10] IO_PU_O									//
//------------------------------------------------------------------------------//

	wire		[13:0] IO_PU_O ;

	assign	IO_PU_O[10]	= (L_125GEN) ? 1'bZ : IO_PU_O_M[10];
	assign	IO_PU_O[11]	= (L_125GEN) ? 1'bZ : IO_PU_O_M[11];
	assign	IO_PU_O[12]	= (L_125GEN) ? 1'bZ : IO_PU_O_M[12];
	assign	IO_PU_O[13]	= (L_125GEN) ? 1'bZ : IO_PU_O_M[13];

	assign	IO_PU_O[9:0]	= IO_PU_O_M  [9:0];



//------------------------------------------------------------------------------//
//			Register`											//
//------------------------------------------------------------------------------//

//------------------------------------------------------------------------------//
//			Data Register  (R/W)									//
//				I/Ox[XAhX +0								//
//				Register		= [13:0]IO_DATAOUT_O				//
//				Register Default 	= 1								//				
// 				Low Rebel			= 0								//
//				Hight Revel		= 1								//
// 																//
// 				I/O Select Register Writer̎ Register data 		//
// 				I/O Select Register Read̎ Register data ǂݍ		//
// 																//
//------------------------------------------------------------------------------//

// LANZXCompactFlash Specification Revision 1.4 ɏĂ܂B
// ڍׂɂĂ͐发ГłmFB

// INDEX0W	= Data Register [hANZX
// INDEX0BL	= Data Register oCgANZXioCgANZXj
// INDEX0BH	= Data Register oCgANZXioCgANZXj
// INDEX0BHBA	= Data Register nCoCgANZXioCgANZXj

	wire		INDEX0W;
	assign	INDEX0W = (IO_C_ADR_I[3:1]	== 3'b000)
						&&(IO_nCE_I	== 2'b00);
 
	wire		INDEX0BL;
	assign	INDEX0BL =(IO_C_ADR_I[3:0]	== 4'b0000)
						&&(IO_nCE_I	== 2'b10);

	wire		INDEX0BH;
	assign	INDEX0BH = (IO_C_ADR_I[3:0]	== 4'b0001)
						 &&(IO_nCE_I	== 2'b10);

	wire		INDEX0BHBA;
	assign	INDEX0BHBA = (IO_C_ADR_I[3:1]	== 3'b000)
						 &&(IO_nCE_I	== 2'b01);

//------------------------------------------------------------------------------//

	always @(posedge ENIO_nIOWR_I or posedge MRST) begin

		if (MRST == 1) begin
			IO_DATAOUT_O[13:0]	<=	14'b11_1111_1111_1111;

		end

		else if (MRST	== 0) begin					
			
			if (IO_nREG_I	== 1'b0) begin 

				if (INDEX0W) begin
					IO_DATAOUT_O[13:0]	<=	C_DATAIN_I[13:0];

				end if (INDEX0BL) begin			
					IO_DATAOUT_O[7:0]	<=	C_DATAIN_I[7:0];
					IO_DATAOUT_O[13:8]	<=	IO_DATAOUT_O[13:8];

				end if (INDEX0BH) begin
					IO_DATAOUT_O[7:0]	<=	IO_DATAOUT_O[7:0];
					IO_DATAOUT_O[13:8]	<=	C_DATAIN_I[5:0];

				end if (INDEX0BHBA) begin
					IO_DATAOUT_O[7:0]	<=	IO_DATAOUT_O[7:0];
					IO_DATAOUT_O[13:8]	<=	C_DATAIN_I[13:8];

				end
			end
		end
	end

//------------------------------------------------------------------------------//
//			I/O Select Register (R/W)								//
//				I/Ox[XAhX +2								//
//				Register		= [13:0]IO_DATAOUT_EN_O				//
//				Register Default 	= 1								//				
// 				Write Mode Set		= 0								//
//				Read Mode Set		= 1								//
// 																//
//------------------------------------------------------------------------------//

// LANZXCompactFlash Specification Revision 1.4 ɏĂ܂B
// ڍׂɂĂ͐发ГłmFB

// INDEX2W	= I/O Select Register [hANZX
// INDEX2BL	= I/O Select Register oCgANZXioCgANZXj
// INDEX2BH	= I/O Select Register oCgANZXioCgANZXj
// INDEX2BHBA	= I/O Select Register nCoCgANZXioCgANZXj

	wire		INDEX2W;								
	assign	INDEX2W = (IO_C_ADR_I[3:1]	==	3'b001)
						&&(IO_nCE_I	==	2'b00);

	wire		INDEX2BL;								
	assign	INDEX2BL =(IO_C_ADR_I[3:0]	==	4'b0010)
						&&(IO_nCE_I	==	2'b10);

	wire		INDEX2BH;								
	assign	INDEX2BH = (IO_C_ADR_I[3:0]	==	4'b0011)
						 &&(IO_nCE_I	==	2'b10);

	wire		INDEX2BHBA;						
	assign	INDEX2BHBA = (IO_C_ADR_I[3:1]	==	4'b001)
						 &&(IO_nCE_I	==	2'b01);

//------------------------------------------------------------------------------//

	always @(posedge ENIO_nIOWR_I or posedge MRST) begin

		if (MRST == 1) begin
			IO_DATAOUT_EN_O[13:0]	<=	14'b11_1111_1111_1111;

		end

		else if (MRST == 0) begin					
		 	if (IO_nREG_I == 1'b0) begin 

				if (INDEX2W) begin					
					IO_DATAOUT_EN_O[13:0]	<=	C_DATAIN_I[13:0];

				end if (INDEX2BL) begin
					IO_DATAOUT_EN_O[7:0]	<=	C_DATAIN_I[7:0];
					IO_DATAOUT_EN_O[13:8]	<=	IO_DATAOUT_EN_O[13:8];

				end if (INDEX2BH) begin
					IO_DATAOUT_EN_O[7:0]	<=	IO_DATAOUT_EN_O[7:0];
					IO_DATAOUT_EN_O[13:8]	<=	C_DATAIN_I[5:0];

				end if (INDEX2BHBA) begin
					IO_DATAOUT_EN_O[13:8]	<=	C_DATAIN_I[13:8];
					IO_DATAOUT_EN_O[7:0]	<=	IO_DATAOUT_EN_O[7:0];
				end
			end
		end
	end

//------------------------------------------------------------------------------//
//			Output Mode Register (W)									//
//				I/Ox[XAhX +4								//
//				Register		= [13:0]IO_DATAOUT_SEL_O				//
//				Register Default 	= 1								//				
// 				Open Drain		= 0								//
//				Totem Pole 		= 1								//
// 																//
//------------------------------------------------------------------------------//

// LANZXCompactFlash Specification Revision 1.4 ɏĂ܂B
// ڍׂɂĂ͐发ГłmFB

// INDEX4W	= Output Mode Register [hANZX
// INDEX4BL	= Output Mode Register oCgANZXioCgANZXj
// INDEX4BH	= Output Mode Register oCgANZXioCgANZXj
// INDEX4BHBA	= Output Mode Register nCoCgANZXioCgANZXj

	wire		INDEX4W;
	assign	INDEX4W = (IO_C_ADR_I[3:1]	==	3'b010)
						&&(IO_nCE_I	==	2'b00);

	wire		INDEX4BL;
	assign	INDEX4BL =(IO_C_ADR_I[3:0]	==	4'b0100)
						&&(IO_nCE_I	==	2'b10);

	wire		INDEX4BH;
	assign	INDEX4BH = (IO_C_ADR_I[3:0]	==	4'b0101)
						 &&(IO_nCE_I	==	2'b10);

	wire		INDEX4BHBA;
	assign	INDEX4BHBA = (IO_C_ADR_I[3:1]	==	3'b010)
						 &&(IO_nCE_I	==	2'b01);


	always @(posedge ENIO_nIOWR_I or posedge MRST) begin

		if (MRST == 1) begin
			IO_DATAOUT_SEL_O[13:0]	<=	14'b11_1111_1111_1111;

		end

		else if (MRST	== 0) begin							
			if (IO_nREG_I	== 1'b0) begin 

				if	(INDEX4W)	begin					
					IO_DATAOUT_SEL_O[13:0]	<=	C_DATAIN_I[13:0];

				end if (INDEX4BL) begin
					IO_DATAOUT_SEL_O[7:0]	<=	C_DATAIN_I[7:0];
					IO_DATAOUT_SEL_O[13:8]	<=	IO_DATAOUT_SEL_O[13:8];

				end if (INDEX4BH) begin
					IO_DATAOUT_SEL_O[7:0]	<=	IO_DATAOUT_SEL_O[7:0];
					IO_DATAOUT_SEL_O[13:8]	<=	C_DATAIN_I[5:0];

				end if (INDEX4BHBA) begin
					IO_DATAOUT_SEL_O[7:0]	<=	IO_DATAOUT_SEL_O[7:0];
					IO_DATAOUT_SEL_O[13:8]	<=	C_DATAIN_I[13:8];
								
				end
			end
		end
	end

//------------------------------------------------------------------------------//
//				PU/PD Select Register (W)							//
//				I/Ox[XAhX +6								//
//				Register		= [13:0]IO_PU_O_M					//
//				Register Default 	= 1								//				
// 				Pull Downs		= 0								//
//				Pull Ups	 		= 1								//
// 																//
//------------------------------------------------------------------------------//

// LANZXCompactFlash Specification Revision 1.4 ɏĂ܂B
// ڍׂɂĂ͐发ГłmFB

// INDEX6W	= PU/PD Select Register [hANZX
// INDEX6BL	= PU/PD Select Register oCgANZXioCgANZXj
// INDEX6BH	= PU/PD Select Register oCgANZXioCgANZXj
// INDEX6BHBA	= PU/PD Select Register nCoCgANZXioCgANZXj

	wire		INDEX6W;
	assign	INDEX6W = (IO_C_ADR_I[3:1]	==	3'b011)
						&&(IO_nCE_I	==	2'b00);

	wire		INDEX6BL;
	assign	INDEX6BL =(IO_C_ADR_I[3:0]	==	4'b0110)
						&&(IO_nCE_I	==	2'b10);

	wire		INDEX6BH;
	assign	INDEX6BH = (IO_C_ADR_I[3:0]	==	4'b0111)
						&&(IO_nCE_I	==	2'b10);

	wire		INDEX6BHBA;
	assign	INDEX6BHBA = (IO_C_ADR_I[3:1]	==	3'b011)
						&&(IO_nCE_I	==	2'b01);

//------------------------------------------------------------------------------//

	always @(posedge ENIO_nIOWR_I or posedge MRST) begin

		if (MRST == 1) begin
			IO_PU_O_M[13:0]	<=	14'b11_1111_1111_1111;

		end else if (MRST	== 0) begin					
			
			if (IO_nREG_I	== 1'b0) begin

				if (INDEX6W) begin
					IO_PU_O_M[13:0]	<=	C_DATAIN_I[13:0];

				end if (INDEX6BL) begin
					IO_PU_O_M[7:0]	<=	C_DATAIN_I[7:0];
					IO_PU_O_M[13:8]	<=	IO_PU_O_M[13:8];

				end if (INDEX6BH) begin
					IO_PU_O_M[7:0]	<=	IO_PU_O_M[7:0];
					IO_PU_O_M[13:8]	<=	C_DATAIN_I[5:0];	

				end if (INDEX6BHBA) begin
					IO_PU_O_M[7:0]	<=	IO_PU_O_M[7:0];
					IO_PU_O_M[13:8]	<=	C_DATAIN_I[13:8];						

			end
			end
		end
	end


//------------------------------------------------------------------------------//


endmodule
